/**
 * GamePanel.java
 * Main game panel
 *
 * @author  Paul Krishnamurthy
 */

package com.example.flappybird;

import com.example.flappybird.states.GameState;
import com.example.flappybird.states.MenuState;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class GamePanel extends JPanel implements KeyListener, MouseListener {
	// Textures
	private GameState gameState;

	// If game has loaded
	public boolean ready = false;

	public GamePanel () {
		setGameState(new MenuState(this));

		// Make the panel focusable
		setFocusable(true);
		requestFocusInWindow();

		gameState.restart(); // Reset game variables

		// Input listeners
		addKeyListener(this);
		addMouseListener(this);
	}

	/**
	 * Sets the current game state.
	 * @param state New GameState
	 */
	public void setGameState(GameState state) {
		System.out.println("Switching to state: " + state.getClass().getName());
		this.gameState = state;
		repaint();
	}

	/**
	 * To start game after everything has been loaded
	 */
	public void addNotify() {
		super.addNotify();
		requestFocusInWindow(); // Ensures the panel gains focus
		ready = true;
	}

	@Override
	public void paintComponent (Graphics g) {
		super.paintComponent(g);
		gameState.renderGameState(g);
	}

	//////////////////////
	// Keyboard actions //
	//////////////////////

	public void keyTyped (KeyEvent e) {}
	public void keyReleased (KeyEvent e) {}

	public void keyPressed(KeyEvent e) {
		System.out.println("Key pressed: " + e.getKeyCode());
		gameState.handleKeyboardEvent(e);
	}

	///////////////////
	// Mouse actions //
	///////////////////

	public void mouseExited (MouseEvent e) {}
	public void mouseEntered (MouseEvent e) {}
	public void mouseReleased (MouseEvent e) {}
	public void mouseClicked (MouseEvent e) {}

	public void mousePressed(MouseEvent e) {
		System.out.println("Mouse clicked at: " + e.getPoint());
		gameState.handleMouseEvent(e);
	}
}

