package com.example.flappybird.states;

import com.example.flappybird.FlappyBird;
import com.example.flappybird.GamePanel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.net.URI;

public class MenuState extends GameState {
    public MenuState(GamePanel panel) {
        super(panel);
    }

    @Override
    public void renderGameState(Graphics g) {
        prepareScreen(g);

        drawBase(g);
        drawMenu(g);

        gameBird.menuFloat();
    }

    /**
     * Draws a string centered based on given restrictions
     *
     * @param s     String to be drawn
     * @param w     Constraining width
     * @param h     Constraining height
     * @param y     Fixed y-coordiate
     */
    public void drawCentered (String s, int w, int h, int y, Graphics g) {
        FontMetrics fm = g.getFontMetrics();

        // Calculate x-coordinate based on string length and width
        int x = (w - fm.stringWidth(s)) / 2;
        g.drawString(s, x, y);
    }

    protected void drawMenu (Graphics g) {
        // Title
        g.drawImage(textures.get("titleText").getImage(),
                textures.get("titleText").getX(),
                textures.get("titleText").getY(), null);

        // Buttons
        g.drawImage(textures.get("playButton").getImage(),
                textures.get("playButton").getX(),
                textures.get("playButton").getY(), null);
        g.drawImage(textures.get("leaderboard").getImage(),
                textures.get("leaderboard").getX(),
                textures.get("leaderboard").getY(), null);
        g.drawImage(textures.get("rateButton").getImage(),
                textures.get("rateButton").getX(),
                textures.get("rateButton").getY(), null);

        // Credits :p
        drawCentered("Created by Paul Krishnamurthy", FlappyBird.WIDTH, FlappyBird.HEIGHT, 600, g);
        g.setFont(flappyMiniFont); // Change font
        drawCentered("www.PaulKr.com", FlappyBird.WIDTH, FlappyBird.HEIGHT, 630, g);
    }

    /**
     * Tries to open the review url in default web browser
     */
    public void openReviewUrl() {

        try {
            if (Desktop.isDesktopSupported()) {
                Desktop.getDesktop().browse(new URI("http://paulkr.com"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("Sorry could not open URL...");
        }

    }

    @Override
    public void handleKeyboardEvent(KeyEvent e) {
        int keyCode = e.getKeyCode();
        // Start game on 'enter' key
        if (keyCode == KeyEvent.VK_ENTER) {
            gamePanel.setGameState(new PlayState(gamePanel));
        }
    }

    @Override
    public void handleMouseEvent(MouseEvent e) {
        // Save clicked point
        clickedPoint = e.getPoint();

        if (isTouching(GameState.textures.get("playButton").getRect())) {
            gamePanel.setGameState(new PlayState(gamePanel));

        } else if (isTouching(GameState.textures.get("leaderboard").getRect())) {
            // Dummy message
            JOptionPane.showMessageDialog(gamePanel,
                    "We can't access the leaderboard right now!",
                    "Oops!",
                    JOptionPane.ERROR_MESSAGE);
        }

        Rectangle rect = GameState.textures.get("playButton").getRect();
        System.out.println("Texture Rect: x=" + rect.getX() + ", y=" + rect.getY() +
                ", width=" + rect.getWidth() + ", height=" + rect.getHeight());

        if (gameBird.isAlive()) {
            if (isTouching(textures.get("rateButton").getRect())) {
                openReviewUrl(); // Open website
            }
        }
    }
}
